@@ -445,7 +445,7 @@ def test_log_prediction_event_invalid_ignore_inputs(ignore_inputs, cli_obj, capl
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize("feedback_id", [[10, 20], "foo", ["foo"], 10, 10.5])
+@pytest.mark.parametrize("feedback_id", [[10, 20], ["foo"], 10, 10.5])
 def test_log_prediction_event_invalid_feedback_id(feedback_id, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_prediction_event(
@@ -761,7 +761,7 @@ def test_log_record_invalid_ignore_inputs(ignore_inputs, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize("feedback_id", [[10, 20], "foo", ["foo"], 10, 10.5])
+@pytest.mark.parametrize("feedback_id", [[10, 20], ["foo"], 10, 10.5])
 def test_log_record_invalid_feedback_id_for_prediction(feedback_id, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_record(
@@ -778,6 +778,86 @@ def test_log_record_invalid_feedback_id_for_prediction(feedback_id, cli_obj, cap
     assert "TypeError:" in caplog.text
 
 
+@freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("feedback_id", ["foo", {"foo": "bar"}])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_log_record_valid_feedback_id_for_prediction(mock_uuid, feedback_id, cli_obj):
+    mock_uuid.return_value = "12345"
+    cli_obj.log_record(
+        application="some-name",
+        inputs={"A": 100},
+        outputs={"value": "bar"},
+        version="a.b.c",
+        ignore_inputs=["B"],
+        feedback_id=feedback_id,
+        timestamp=SOME_TIME,
+    )
+    assert cli_obj.log_store.logs == [
+        {
+            "events": [
+                {
+                    "batch_id": None,
+                    "event_id": "12345",
+                    "inputs": {"A": 100},
+                    "outputs": {"value": "bar"},
+                    "tags": {"env": "test"},
+                    "feedback_id": "42a5b7b99b3717d1eaeb72a6948dabc9"
+                    if isinstance(feedback_id, dict)
+                    else feedback_id,
+                    "log_timestamp": CURRENT_TIME_STR,
+                    "metadata": {
+                        "environment": "test",
+                        "func_name": "some-name",
+                        "provided_feedback_id": feedback_id,
+                        "version": "a.b.c",
+                        "ignore_inputs": ["B"],
+                        "feedback_keys": None,
+                    },
+                    "timestamp": SOME_TIME_STR,
+                }
+            ],
+            "application": "some-name",
+        },
+    ]
+
+
+@freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("feedback_id", ["foo", {"foo": "bar"}])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_log_record_valid_feedback_id_for_feedback(mock_uuid, feedback_id, cli_obj):
+    mock_uuid.return_value = "12345"
+    cli_obj.log_record(
+        application="some-name",
+        feedback={"A": 100},
+        version="a.b.c",
+        feedback_id=feedback_id,
+        timestamp=SOME_TIME,
+    )
+    assert cli_obj.log_store.logs == [
+        {
+            "events": [
+                {
+                    "batch_id": None,
+                    "event_id": "12345",
+                    "feedback": {"A": 100},
+                    "feedback_id": "42a5b7b99b3717d1eaeb72a6948dabc9"
+                    if isinstance(feedback_id, dict)
+                    else feedback_id,
+                    "feedback_id_inputs": feedback_id,
+                    "log_timestamp": CURRENT_TIME_STR,
+                    "metadata": {
+                        "environment": "test",
+                        "func_name": "some-name",
+                        "feedback_version": "a.b.c",
+                    },
+                    "timestamp": SOME_TIME_STR,
+                }
+            ],
+            "application": "some-name",
+        },
+    ]
+
+
 @pytest.mark.parametrize("timestamp", [[], "some-timestamp", 10, "2020-10-10"])
 def test_log_record_invalid_timestamp_for_prediction(timestamp, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
