@@ -37,9 +37,9 @@ def test_metric_query_invalid_num_points(num_points, gantry_metric_obj, series_o
         )
 
 
-def test_metric_query(gantry_metric_obj, series_obj_factory_with_filters):
-    series_obj_1 = series_obj_factory_with_filters("float", [1, 2, 3])
-    series_obj_2 = series_obj_factory_with_filters("bool", [1, 2, 3])
+def test_metric_query(gantry_metric_obj, series_obj_factory_with_filters_and_tags):
+    series_obj_1 = series_obj_factory_with_filters_and_tags("float", [1, 2, 3], {"foo": "bar"})
+    series_obj_2 = series_obj_factory_with_filters_and_tags("bool", [1, 2, 3], None)
     data = {
         "start_time": to_tz_time(START_TIME),
         "end_time": to_tz_time(END_TIME),
