@@ -2,6 +2,7 @@ import datetime
 from collections import defaultdict
 
 import mock
+import numpy as np
 import pandas as pd
 import pytest
 import pytz
@@ -57,23 +58,30 @@ def test_resolve_time_strings_error(start_time, end_time):
         queryframe.GantryQueryFrame.resolve_time_strings(start_time, end_time)
 
 
-@pytest.mark.parametrize("env", [None, "dev"])
+@pytest.mark.parametrize(
+    ["tags", "env", "expected"],
+    [
+        (None, None, None),
+        ({"foo": "bar"}, None, {"foo": "bar"}),
+        ({"foo": "bar"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, None, {"foo": "bar", "env": "another"}),
+        (None, "dev", {"env": "dev"}),
+    ],
+)
 @pytest.mark.parametrize("n", [None, 1, 5])
-def test_populate_raw_request_descending_no_pagination(n, env, api_client_obj):
+def test_populate_raw_request_descending_no_pagination(n, tags, env, expected, api_client_obj):
     query_info_obj = queryframe.QueryInfo(
         "af0d0317-adbf-4524-b838-9dddf1c43fc8", "foobar", "1.2.3", env, START_TIME, END_TIME
     )
     query_frame_obj = queryframe.GantryQueryFrame(api_client_obj, query_info_obj)
 
-    matcher = {}
-    if env:
-        matcher["tags"] = {"env": env}
+    matcher = {"tags": expected} if expected else {}
 
     if n:
         matcher["limit"] = n
 
     matcher["order"] = "descending"
-
     expected_events = list("these-are-events")[:n] if n else list("these-are-events")
     with responses.RequestsMock() as rsps:
         rsps.add(
@@ -92,22 +100,30 @@ def test_populate_raw_request_descending_no_pagination(n, env, api_client_obj):
         )
 
         assert (
-            query_frame_obj._populate_raw_request(query_filters=[], ascending=False, n=n)
+            query_frame_obj._populate_raw_request(query_filters=[], ascending=False, n=n, tags=tags)
             == expected_events
         )
 
 
-@pytest.mark.parametrize("env", [None, "dev"])
+@pytest.mark.parametrize(
+    ["tags", "env", "expected"],
+    [
+        (None, None, None),
+        ({"foo": "bar"}, None, {"foo": "bar"}),
+        ({"foo": "bar"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, None, {"foo": "bar", "env": "another"}),
+        (None, "dev", {"env": "dev"}),
+    ],
+)
 @pytest.mark.parametrize("n", [None, 1, 5])
-def test_populate_raw_request_ascending_no_pagination(n, env, api_client_obj):
+def test_populate_raw_request_ascending_no_pagination(n, tags, env, expected, api_client_obj):
     query_info_obj = queryframe.QueryInfo(
         "af0d0317-adbf-4524-b838-9dddf1c43fc8", "foobar", "1.2.3", env, START_TIME, END_TIME
     )
     query_frame_obj = queryframe.GantryQueryFrame(api_client_obj, query_info_obj)
 
-    matcher = {}
-    if env:
-        matcher["tags"] = {"env": env}
+    matcher = {"tags": expected} if expected else {}
 
     if n:
         matcher["limit"] = n
@@ -132,14 +148,24 @@ def test_populate_raw_request_ascending_no_pagination(n, env, api_client_obj):
         )
 
         assert (
-            query_frame_obj._populate_raw_request(query_filters=[], ascending=True, n=n)
+            query_frame_obj._populate_raw_request(query_filters=[], ascending=True, n=n, tags=tags)
             == expected_events
         )
 
 
-@pytest.mark.parametrize("env", [None, "dev"])
+@pytest.mark.parametrize(
+    ["tags", "env", "expected"],
+    [
+        (None, None, None),
+        ({"foo": "bar"}, None, {"foo": "bar"}),
+        ({"foo": "bar"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, None, {"foo": "bar", "env": "another"}),
+        (None, "dev", {"env": "dev"}),
+    ],
+)
 @pytest.mark.parametrize("n", [10, 53, 101])
-def test_populate_raw_request_ascending_pagination(n, env, api_client_obj):
+def test_populate_raw_request_ascending_pagination(n, tags, env, expected, api_client_obj):
     query_info_obj = queryframe.QueryInfo(
         "af0d0317-adbf-4524-b838-9dddf1c43fc8", "foobar", "1.2.3", env, START_TIME, END_TIME
     )
@@ -151,9 +177,7 @@ def test_populate_raw_request_ascending_pagination(n, env, api_client_obj):
     up_to = n - (n // 2)  # return less than n to test pagination
 
     # Two matchers, one for the first page and one for the second
-    first_matcher = {}
-    if env:
-        first_matcher["tags"] = {"env": env}
+    first_matcher = {"tags": expected} if expected else {}
     first_matcher["limit"] = n
     first_matcher["order"] = "ascending"
 
@@ -196,14 +220,24 @@ def test_populate_raw_request_ascending_pagination(n, env, api_client_obj):
             match=[matchers.json_params_matcher(second_matcher)],
         )
         assert (
-            query_frame_obj._populate_raw_request(query_filters=[], ascending=True, n=n)
+            query_frame_obj._populate_raw_request(query_filters=[], ascending=True, n=n, tags=tags)
             == requested_events
         )
 
 
-@pytest.mark.parametrize("env", [None, "dev"])
+@pytest.mark.parametrize(
+    ["tags", "env", "expected"],
+    [
+        (None, None, None),
+        ({"foo": "bar"}, None, {"foo": "bar"}),
+        ({"foo": "bar"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, "dev", {"foo": "bar", "env": "dev"}),
+        ({"foo": "bar", "env": "another"}, None, {"foo": "bar", "env": "another"}),
+        (None, "dev", {"env": "dev"}),
+    ],
+)
 @pytest.mark.parametrize("n", [10, 53, 101])
-def test_populate_raw_request_descending_pagination(n, env, api_client_obj):
+def test_populate_raw_request_descending_pagination(n, tags, env, expected, api_client_obj):
     query_info_obj = queryframe.QueryInfo(
         "af0d0317-adbf-4524-b838-9dddf1c43fc8", "foobar", "1.2.3", env, START_TIME, END_TIME
     )
@@ -215,9 +249,7 @@ def test_populate_raw_request_descending_pagination(n, env, api_client_obj):
     up_to = n - (n // 2)  # return less than n to test pagination
 
     # Two matchers, one for the first page and one for the second
-    first_matcher = {}
-    if env:
-        first_matcher["tags"] = {"env": env}
+    first_matcher = {"tags": expected} if expected else {}
     first_matcher["limit"] = n
     first_matcher["order"] = "descending"
 
@@ -260,20 +292,24 @@ def test_populate_raw_request_descending_pagination(n, env, api_client_obj):
             match=[matchers.json_params_matcher(second_matcher)],
         )
         assert (
-            query_frame_obj._populate_raw_request(query_filters=[], ascending=False, n=n)
+            query_frame_obj._populate_raw_request(query_filters=[], ascending=False, n=n, tags=tags)
             == requested_events
         )
 
 
+@pytest.mark.parametrize("provided_tags", [None, {}, {"foo": "bar"}])
 @pytest.mark.parametrize("num_events", [1, 5, 10, 20])
 @pytest.mark.parametrize("column", ["A", None, "not_a_column"])
 @pytest.mark.parametrize("method", ["_head", "_tail"])
 @mock.patch("gantry.query.core.queryframe.GantryQueryFrame._populate_raw_request")
 @mock.patch("gantry.query.core.dataframe.GantryQueryFrame._dtypes", new_callable=mock.PropertyMock)
-def test_head_tail(mock_dtypes, mock_populate_raw, method, column, num_events, query_frame_obj):
+def test_head_tail(
+    mock_dtypes, mock_populate_raw, method, column, num_events, provided_tags, query_frame_obj
+):
     columns = ["A", "B", "C"]
 
-    def fake_populate_raw(n, ascending, query_filters):
+    def fake_populate_raw(n, ascending, query_filters, tags):
+        assert provided_tags == tags
         events = defaultdict(list)
         for c in columns:
             if c != "not_a_column":
@@ -285,10 +321,10 @@ def test_head_tail(mock_dtypes, mock_populate_raw, method, column, num_events, q
 
     if column == "not_a_column":
         with pytest.raises(ValueError):
-            _ = getattr(query_frame_obj, method)(n=num_events, column=column)
+            _ = getattr(query_frame_obj, method)(n=num_events, column=column, tags=provided_tags)
 
     else:
-        df = getattr(query_frame_obj, method)(n=num_events, column=column)
+        df = getattr(query_frame_obj, method)(n=num_events, column=column, tags=provided_tags)
 
         expected = []
         for i in range(num_events):
@@ -300,14 +336,16 @@ def test_head_tail(mock_dtypes, mock_populate_raw, method, column, num_events, q
         pd.testing.assert_frame_equal(df, pd.DataFrame(expected))
 
 
+@pytest.mark.parametrize("provided_tags", [None, {}, {"foo": "bar"}])
 @pytest.mark.parametrize("num_events", [1, 5, 10, 20])
 @pytest.mark.parametrize("column", ["A", None, "not_a_column"])
 @mock.patch("gantry.query.core.queryframe.GantryQueryFrame._populate_raw_request")
 @mock.patch("gantry.query.core.dataframe.GantryQueryFrame._dtypes", new_callable=mock.PropertyMock)
-def test_fetch(mock_dtypes, mock_populate_raw, column, num_events, query_frame_obj):
+def test_fetch(mock_dtypes, mock_populate_raw, column, num_events, provided_tags, query_frame_obj):
     columns = ["A", "B", "C"]
 
-    def fake_populate_raw(n, ascending, query_filters):
+    def fake_populate_raw(n, ascending, query_filters, tags):
+        assert tags == provided_tags
         events = defaultdict(list)
         for c in columns:
             if c != "not_a_column":
@@ -319,10 +357,10 @@ def test_fetch(mock_dtypes, mock_populate_raw, column, num_events, query_frame_o
 
     if column == "not_a_column":
         with pytest.raises(ValueError):
-            _ = query_frame_obj._fetch(column=column)
+            _ = query_frame_obj._fetch(column=column, tags=provided_tags)
 
     else:
-        df = query_frame_obj._fetch(column=column)
+        df = query_frame_obj._fetch(column=column, tags=provided_tags)
 
         expected = []
         for i in range(num_events):
@@ -464,10 +502,27 @@ def test_prepare_pandas_dataframe_without_timestamp(mock_dtypes, query_frame_obj
 
 
 @pytest.mark.parametrize("column", [None, "A"])
-def test_prepare_pandas_dataframe_empty(column, query_frame_obj):
+@mock.patch("gantry.query.core.dataframe.GantryQueryFrame._dtypes", new_callable=mock.PropertyMock)
+def test_prepare_pandas_dataframe_empty(mock_dtypes, column, query_frame_obj):
+    _dtypes = {
+        "A": "bool",
+        "B": "category",
+        "C": "float",
+    }
+    mock_dtypes.return_value = pd.Series(_dtypes)
     df = query_frame_obj._prepare_pandas_dataframe([], column=column)
     assert df.empty
 
+    assert len(df.columns) == 3
+
+    assert df.columns[0] == "A"
+    assert df.columns[1] == "B"
+    assert df.columns[2] == "C"
+
+    assert df.dtypes[0] == "bool"
+    assert df.dtypes[1] == "category"
+    assert df.dtypes[2] == np.dtype("float64")
+
 
 def test_metadata(query_frame_obj):
     with responses.RequestsMock() as rsps:
